/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SceneDropTarget.java,v 4a66a2688ef2 2010/03/12 01:27:41 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.visual.impl;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDropEvent;
import org.openide.nodes.Node;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.visual.DropAcceptor;
import it.tidalwave.netbeans.visual.NodeScene;
import org.netbeans.api.visual.model.ObjectScene;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: SceneDropTarget.java,v 4a66a2688ef2 2010/03/12 01:27:41 fabrizio $
 *
 **********************************************************************************************************************/
public class SceneDropTarget extends DropTarget
  {
    private static final String CLASS = SceneDropTarget.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final ObjectScene scene;

    private final Map<DataFlavor, DropAcceptor> dropAcceptorMap = new HashMap<DataFlavor, DropAcceptor>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public SceneDropTarget (final @Nonnull ObjectScene scene)
      {
        this.scene = scene;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void registerDropAcceptor (final @Nonnull DataFlavor dataFlavor,
                                      final @Nonnull DropAcceptor dropAcceptor)
      {
        dropAcceptorMap.put(dataFlavor, dropAcceptor);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void drop (final @Nonnull DropTargetDropEvent event)
      {
        logger.fine("drop(%s)", event);

        try
          {
            for (final DataFlavor flavor : event.getCurrentDataFlavors())
              {
                final DropAcceptor dropAcceptor = dropAcceptorMap.get(flavor);
                logger.finer(">>>> trying flavor: %s, dropAcceptor: %s", flavor, dropAcceptor);

                if (dropAcceptor != null)
                  {
                    event.acceptDrop(event.getDropAction());
                    final Object object = event.getTransferable().getTransferData(flavor);
                    logger.finer(">>>> object is: " + object);

                    if (object != null)
                      {
                        final Collection<Node> nodes = dropAcceptor.process(event, object);

                        if (!nodes.isEmpty())
                          {
                            int i = 0;

                            for (final Node node : nodes)
                              {
                                final int delta = (50 * i) / Math.max(1, nodes.size() - 1);
                                final Point dropLocation = event.getLocation();

                                if (scene instanceof NodeScene)
                                  {
                                    ((NodeScene)scene).addNode(node, new Point(dropLocation.x + delta, dropLocation.y + delta));
                                  }
                                else
                                  {
//                                    scene.addNode(node); FIXME
                                  }
                              }

                            logger.finer(">>>> drop accepted");
                            event.dropComplete(true);
                            return;
                          }
                      }
                  }
              }
          }
        catch (Exception e)
          {
            logger.throwing(CLASS, "drop()", e);
          }

        logger.finer(">>>> drop rejected");
        event.dropComplete(false);
      }
  }
